import 'dart:async';

import 'package:flutter_data/flutter_data.dart';
import 'package:flutter_data_json_api_adapter/flutter_data_json_api_adapter.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:freezed_annotation/freezed_annotation.dart';

part 'models.freezed.dart';
part 'models.g.dart';

@JsonSerializable()
@DataRepository([JSONAPIAdapter, TestMixin])
class Model with DataModel<Model> {
  @override
  final String id;
  final String name;
  final BelongsTo<Company> company;

  Model({this.id, this.name, this.company});
}

@freezed
@DataRepository([JSONAPIAdapter, TestMixin])
abstract class City with DataModel<City>, _$City {
  City._();
  factory City({
    String id,
    String name,
  }) = _City;

  factory City.fromJson(Map<String, dynamic> json) => _$CityFromJson(json);
}

@freezed
@DataRepository([JSONAPIAdapter, TestMixin])
abstract class Company with DataModel<Company>, _$Company {
  Company._();
  factory Company({
    String id,
    String name,
    String nasdaq,
    DateTime updatedAt,
    HasMany<Model> models,
  }) = _Company;

  factory Company.fromJson(Map<String, dynamic> json) =>
      _$CompanyFromJson(json);
}

//

mixin TestMixin<T extends DataModel<T>> on RemoteAdapter<T> {
  @override
  String get baseUrl => 'http://127.0.0.1:17083/';

  @override
  FutureOr<Map<String, dynamic>> get params => {
        'page': {'limit': 10}
      };

  @override
  FutureOr<Map<String, String>> get headers => {'x-client-id': '2473272'};

  // test random method
  Future<T> loginTest(T model) {
    return null;
  }

  // @override
  // FutureOr<R> withRequest<R>(String url,
  //     {DataRequestMethod method,
  //     Map<String, dynamic> params,
  //     Map<String, String> headers,
  //     String body,
  //     OnData<R> onSuccess,
  //     OnData<R> onError}) {
  //   return super.withRequest(
  //     url,
  //     method: method,
  //     params: params,
  //     headers: headers,
  //     body: body,
  //     onSuccess: (data) {
  //       if (type == 'models') {
  //         // if it's ONE model of type models and id is long (i.e. autogenerated)
  //         if (data['data'] is! List &&
  //             data['data']['type'] == 'models' &&
  //             data['data']['id'].toString().length > 6) {
  //           data['data']['id'] = '9217';
  //         }
  //       }
  //       return onSuccess(data);
  //     },
  //   );
  // }
}
